#include <iomanip>
#include <iostream>
#include <limits>
#include <complex>
#include <time.h>

#include "tbb/tick_count.h"
#include "tbb/task_scheduler_init.h"
#include "tbb/blocked_range.h"
#include "tbb/parallel_for.h"

using namespace tbb; 
using namespace std; 

#define PI (3.14159265358979323846)

void DummyDataInitialization(complex<double>* mas, int size)  {
  for(int i=0; i<size; i++) 
    mas[i]=0 ;

  mas[size-size/4]=1;
}

void RandomDataInitialization(complex<double>* mas, int size)  {
  srand(unsigned(clock()));
  for(int i=0; i<size; i++) 
    mas[i]=complex<double>(rand()/1000.0, rand()/1000.0) ;
}

void Initialize(complex<double>* &inputSignal, complex<double>* &outputSignal, int &size)  {
  do {
    cout << "Enter the input signal length: ";
    cin >> size;

    if(size < 4) 
      cout << "Input signal length should be greater or equal than four" << endl;
    else {
      int tmpSize=size;

      while(tmpSize!=1) {
        if(tmpSize%2!=0) {
          cout << "Input signal length should be powers of two" << endl;
          size=-1;
          break;
        }
        tmpSize/=2;
      }
    }
  }
  while(size < 4);

  inputSignal=new complex<double>[size];
  //Computing experiments
  //RandomDataInitialization(inputSignal, size);
  //-------------------------------------------------

  //Tests
  DummyDataInitialization(inputSignal, size);
  //-------------------------------------------------

  outputSignal=new complex<double>[size];
}

void Terminate(complex<double>* &inputSignal, complex<double>* &outputSignal) {
  delete[] inputSignal;
  inputSignal=NULL;

  delete[] outputSignal;
  outputSignal=NULL;
}

void BitReversing(complex<double> *inputSignal, complex<double> *outputSignal, int size) {
  int j = 0, i = 0;

  while (i < size) {
    if (j > i) {
      outputSignal[i] = inputSignal[j];
      outputSignal[j] = inputSignal[i];
    }
    else
      if (j == i) 
        outputSignal[i] = inputSignal[i];

    int m = size >> 1;
    while ((m >= 1) && (j >= m)) {
      j -= m ;
      m = m >> 1;
    }
    j += m;
    i ++;
  }
}

__inline void Butterfly(complex<double> *signal, complex<double> &u, int offset, int butterflySize) {
  complex<double> tem = signal[offset +  butterflySize] * u;

  signal[offset + butterflySize] = signal[offset] - tem;
  signal[offset] += tem;
}

void ParallelFFTCalculation(complex<double> *signal, int size, int grainSize) {
  //...
}

void ParallelFFT(complex<double> *inputSignal, complex<double> *outputSignal, int size, int grainSize)  {
  BitReversing(inputSignal, outputSignal, size);
  ParallelFFTCalculation(outputSignal, size, grainSize);
}

void PrintSignal(complex<double> *signal, int size)
{
  cout << fixed << setprecision(0);
  for(int i=0; i<size; i++) {
    cout << signal[i] << endl;
  }
}

void PrintAmplitude(complex<double> *signal, int size)
{
  for(int i=0; i<size; i++) {
    cout << abs(signal[i]) << endl;
  }
}

int main() { 
  complex<double> *inputSignal=NULL;
  complex<double> *outputSignal=NULL;

  int size=0;

  cout << "Fast Fourier Transform. Parallel version." << endl;

  Initialize(inputSignal, outputSignal, size);
  
  //...
  
  Terminate(inputSignal, outputSignal);
  cin.get();

  return 0; 
} 